using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace QiHe.CodeLib
{
    public class DataReader
    {
        Stream baseStream;

        public Stream BaseStream
        {
            get { return baseStream; }
            set { baseStream = value; }
        }

        public DataReader(Stream stream)
        {
            baseStream = stream;
        }

        public byte ReadByte()
        {
            int value = baseStream.ReadByte();
            if (value == -1) throw new EndOfStreamException();
            return (byte)value;
        }

        public UInt32 ReadUInt32LE()
        {
            byte[] bytes = new byte[4];
            int num = baseStream.Read(bytes, 0, 4);
            if (num < 4) throw new EndOfStreamException();
            UInt32 value = (UInt32)(((((bytes[3] << 8) + bytes[2]) << 8) + bytes[1]) << 8) + bytes[0];
            return value;
        }

        public UInt16 ReadUInt16BE()
        {
            byte[] bytes = new byte[2];
            int num = baseStream.Read(bytes, 0, 2);
            if (num < 2) throw new EndOfStreamException();
            UInt16 value = (UInt16)((bytes[0] << 8) + bytes[1]);
            return value;
        }

        public UInt32 ReadUInt32BE()
        {
            byte[] bytes = new byte[4];
            int num = baseStream.Read(bytes, 0, 4);
            if (num < 4) throw new EndOfStreamException();
            UInt32 value = (uint)(((((bytes[0] << 8) + bytes[1]) << 8) + bytes[2]) << 8) + bytes[3];
            return value;
        }

        public byte[] ReadBytes(int count)
        {
            int available = (int)(baseStream.Length - baseStream.Position);
            if (available < count) throw new EndOfStreamException();
            byte[] bytes = new byte[count];
            int num = baseStream.Read(bytes, 0, count);
            if (num < count) throw new EndOfStreamException();
            return bytes;
        }
    }
}
