using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace QiHe.CodeLib
{
    public class OverlayStream : Stream
    {
        Stream baseStream;
        long startPosition;
        long length;

        public OverlayStream(Stream stream, long start)
        {
            this.baseStream = stream;
            this.startPosition = start;
            this.length = stream.Length - start;
        }

        public OverlayStream(Stream stream, long start, long length)
        {
            this.baseStream = stream;
            this.startPosition = start;
            this.length = length;
            if (length > stream.Length - start)
            {
                this.length = stream.Length - start;
                //throw new EndOfStreamException("OverlayStream's length can't exceed BaseStream's length.");
            }
        }

        public override long Position
        {
            get { return baseStream.Position - startPosition; }
            set { baseStream.Position = startPosition + value; }
        }

        public override long Length
        {
            get { return length; }
        }

        public override void SetLength(long value)
        {
            length = value;
        }

        public override bool CanRead
        {
            get { return baseStream.CanRead; }
        }

        public override bool CanWrite
        {
            get { return baseStream.CanWrite; }
        }

        public override bool CanSeek
        {
            get { return baseStream.CanSeek; }
        }

        public override void Flush()
        {
            baseStream.Flush();
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            return baseStream.Seek(offset - startPosition, origin);
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            int available = count;
            if (Position + count > Length)
            {
                available = (int)(Length - Position);
            }
            return baseStream.Read(buffer, offset, available);
        }

        public byte[] ReadToEnd()
        {
            int available = (int)(Length - Position);
            byte[] buffer = new byte[available];
            baseStream.Read(buffer, 0, available);
            return buffer;
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            baseStream.Write(buffer, offset, count);
        }

        public override int ReadByte()
        {
            if (Position < Length)
            {
                return base.ReadByte();
            }
            else
            {
                return -1;
            }
        }
    }
}
