using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Diagnostics;
using System.Reflection;
using System.Drawing.Imaging;

namespace QiHe.CodeLib
{
    /// <summary>
    /// Windows Application Library
    /// </summary>
    public class WinApp
    {
        public static string AppPath
        {
            get
            {
                return Application.StartupPath + Path.DirectorySeparatorChar;
            }
        }
        public static string ConfigFile
        {
            get
            {
                return Path.ChangeExtension(Application.ExecutablePath, ".settings");
            }
        }
        public static T LoadConfig<T>()
        {
            return LoadConfig<T>(ConfigFile);
        }
        public static T LoadConfig<T>(string xmlFile)
        {
            try
            {
                T data = XmlData<T>.Load(xmlFile);
                if (data != null)
                {
                    return data;
                }
            }
            catch { }
            try
            {
                return Activator.CreateInstance<T>();
            }
            catch { }
            return default(T);
        }
        public static void SaveConfig<T>(T data)
        {
            XmlData<T>.Save(ConfigFile, data);
        }
        public static void SaveConfig(object data)
        {
            XmlFile.Save(ConfigFile, data);
        }

        public static void ShowImage(Image bmp, string title)
        {
            Form form = new Form();
            form.Text = title;
            int titleHeight = form.Height - form.ClientSize.Height;
            Rectangle screen = Screen.PrimaryScreen.WorkingArea;
            int width = Math.Min(screen.Width, bmp.Width);
            int height = Math.Min(screen.Height - titleHeight, bmp.Height);
            form.ClientSize = new Size(width, height);
            form.AutoScroll = true;
            form.FormBorderStyle = FormBorderStyle.FixedSingle;
            PictureBox pbx = new PictureBox();
            pbx.Size = bmp.Size;
            pbx.Location = new Point(0, 0);
            pbx.Image = bmp;
            form.Controls.Add(pbx);
            ContextMenu menu = new ContextMenu(
                new MenuItem[]{
                    new MenuItem("Save",new EventHandler(delegate(object sender, EventArgs e)
                {
                    string file = FileSelector.BrowseFileForSave(FileType.Img);
                    if(file != null)
                    {
                        ImageFormat format = ImageFormat.Png;
                        switch(Path.GetExtension(file).ToLower())
                        {
                            case ".jpg":
                                format = ImageFormat.Jpeg;
                                break;
                            case ".gif":
                                format = ImageFormat.Gif;
                                break;
                            case ".emf":
                                format = ImageFormat.Emf;
                                break;
                            case ".bmp":
                                format = ImageFormat.Bmp;
                                break;
                            case ".png":
                                format = ImageFormat.Png;
                                break;
                        }
                        form.BackgroundImage.Save(file, format);
                    }
                }))});
            form.ContextMenu = menu;
            form.BackgroundImage = bmp;
            form.StartPosition = FormStartPosition.CenterScreen;
            form.Show();
        }
        public static void ShowImage(Image bmp, string title, bool scroll)
        {
            Form form = new Form();
            form.Text = title;
            Rectangle screen = Screen.PrimaryScreen.Bounds;
            int width = Math.Min(screen.Width, bmp.Width);
            int height = Math.Min(screen.Height, bmp.Height);
            form.ClientSize = new Size(width, height);
            form.AutoScroll = true;
            form.FormBorderStyle = FormBorderStyle.FixedSingle;
            if (scroll)
            {
                PictureBox pbx = new PictureBox();
                form.Controls.Add(pbx);
                pbx.Size = bmp.Size;
                pbx.Location = new Point(0, 0);
                pbx.Image = bmp;
            }
            else
            {
                form.BackgroundImage = bmp;
            }
            form.Show();
        }
    }
}
