using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace QiHe.RealMedia
{
    public partial class CutterDialog : Form
    {
        public CutterDialog()
        {
            InitializeComponent();
        }

        private void CutterDialog_Load(object sender, EventArgs e)
        {
            comboBoxFrom.SelectedIndex = 0;
            comboBoxTo.SelectedIndex = 0;
        }

        public int StartTime
        {
            get
            {
                return GetTime(textBoxFrom.Text, comboBoxFrom.SelectedItem.ToString());
            }
        }

        public int EndTime
        {
            get
            {
                return GetTime(textBoxTo.Text, comboBoxTo.SelectedItem.ToString());
            }
        }

        private int GetTime(string time, string unit)
        {
            if (unit == "millisecond")
            {
                return int.Parse(time);
            }
            else if (unit == "hh:mm:ss.ms")
            {
                //return (int)TimeSpan.Parse(time).TotalMilliseconds;
                return ParseTime(time);
            }
            else
            {
                throw new FormatException("Invalid time unit format.");
            }
        }

        private int ParseTime(string time)
        {
            int millisecond = 0;
            string[] items = time.Split(':');
            if (items.Length == 1)
            {
                millisecond += GetSecondPart(items[0]);
            }
            else if (items.Length == 2)
            {
                millisecond += GetMinutePart(items[0]);
                millisecond += GetSecondPart(items[1]);
            }
            else if (items.Length == 3)
            {
                millisecond += GetHourPart(items[0]);
                millisecond += GetMinutePart(items[1]);
                millisecond += GetSecondPart(items[2]);
            }
            else
            {
                throw new FormatException("Invalid time format.");
            }
            return millisecond;
        }

        private int GetHourPart(string hour)
        {
            return int.Parse(hour) * 60 * 60 * 1000;
        }

        private static int GetMinutePart(string minute)
        {
            return int.Parse(minute) * 60 * 1000;
        }

        private static int GetSecondPart(string second)
        {
            int millisecond = 0;
            string[] parts = second.Split('.');
            if (parts.Length >= 1)
            {
                millisecond += int.Parse(parts[0]) * 1000;
            }
            if (parts.Length >= 2)
            {
                millisecond += int.Parse(parts[1]);
            }
            return millisecond;
        }
    }
}