using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using QiHe.CodeLib;

namespace QiHe.RealMedia
{
    public class RecordCollection : IEnumerable<Record>, IDisposable
    {
        FormatDef Format;
        RecordDef RecordDef;
        DataReader Reader;
        DataWriter Writer;
        int Size;
        int index;
        long streamPos;
        Stream TempStream;
        string TempFile;

        public int Count
        {
            get { return Size; }
        }

        public RecordCollection(FormatDef format, RecordDef recorddef, DataReader reader, int size)
        {
            Format = format;
            Reader = reader;
            RecordDef = recorddef;
            Size = size;
            index = 0;
            streamPos = Reader.BaseStream.Position;
        }

        public RecordCollection(FormatDef format, RecordDef recorddef, string tempfile)
        {
            Format = format;
            RecordDef = recorddef;
            TempFile = tempfile;
            TempStream = File.Create(tempfile);
            Reader = new DataReader(TempStream);
            Writer = new DataWriter(TempStream);
            streamPos = 0;
            Size = 0;
            index = 0;
        }

        public void Reset()
        {
            index = 0;
            Reader.BaseStream.Position = streamPos;
        }

        public Record Read()
        {
            if (index < Size)
            {
                index++;
                return RecordDef.Decode(Reader, Format);
            }
            else
            {
                return null;
            }
        }

        public void Add(Record record)
        {
            RecordDef.Encode(Writer, record, Format);
            Size++;
        }

        public void AddRange(IEnumerable<Record> records)
        {
            foreach (Record record in records)
            {
                Add(record);
            }
        }

        #region IEnumerable<Record> Members

        IEnumerator<Record> IEnumerable<Record>.GetEnumerator()
        {
            Reset();
            for (int i = 0; i < Size; i++)
            {
                Record record = RecordDef.Decode(Reader, Format);
                yield return record;
                if (record.HasError)
                {
                    break;
                }
            }
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        public override string ToString()
        {
            return RecordDef.Name + " Records";
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (File.Exists(TempFile))
            {
                TempStream.Close();
                File.Delete(TempFile);
            }
        }

        ~RecordCollection()
        {
            Dispose();
        }

        #endregion
    }
}
