using System;
using System.Collections.Generic;
using System.Text;
using QiHe.CodeLib;

namespace QiHe.RealMedia
{
    public class Chunk
    {
        public Int64 Offset;
        public UInt32 ID;
        public UInt32 Size;
        public OverlayStream Data;

        public string Name;
        public Record Record;

        public object this[string fieldname]
        {
            get
            {
                return Record[fieldname];
            }
            set
            {
                Record[fieldname] = value;
            }
        }

        public void Decode(FormatDef format)
        {
            Name = Encoding.ASCII.GetString(BitConverter.GetBytes(ID));
            RecordDef recordDef = format[Name];
            if (recordDef != null)
            {
                Data.Position = 0;
                DataReader reader = new DataReader(Data);
                Record = recordDef.Decode(reader, format);
            }
            else
            {
                Data.Position = 0;
                Record = new Record();
                Record.Name = this.Name;
                Record["Data"] = Data.ReadToEnd();
            }
        }

        public uint CalculateSize(FormatDef format)
        {
            RecordDef recordDef = format[Name];
            if (recordDef != null)
            {
                int datasize = recordDef.CalculateSize(Record, format);
                Size = (uint)(datasize + 8);
            }
            return Size;
        }

        public void Encode(DataWriter writer, FormatDef format)
        {
            writer.WriteUInt32LE(ID);
            writer.WriteUInt32BE(Size);
            RecordDef recordDef = format[Name];
            if (recordDef != null)
            {
                recordDef.Encode(writer, Record, format);
            }
            else
            {
                writer.WriteBytes(Record["Data"] as byte[]);
            }
        }

        public void CopyFrom(Chunk chunk)
        {
            this.ID = chunk.ID;
            this.Name = chunk.Name;
            this.Record = new Record();
            this.Record.CopyFrom(chunk.Record);
        }
    }
}
