using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using QiHe.CodeLib;

namespace QiHe.RealMedia
{
    public class Reader
    {
        public static MediaFile Load(string file, FormatDef format, out bool hasError)
        {
            FileStream stream = File.OpenRead(file);
            DataReader reader = new DataReader(stream);

            MediaFile media = new MediaFile();
            media.FilePath = file;
            media.FileName = Path.GetFileName(file);
            media.FileStream = stream;
            hasError = false;
            while (true)
            {
                try
                {
                    Chunk chunk = DecodeChunk(stream, reader, format);
                    media.Chunks.Add(chunk);
                    if (chunk.Record.HasError)
                    {
                        hasError = true;
                    }
                    if (chunk.Size == 0)
                    {
                        break;
                    }
                }
                catch (EndOfStreamException)
                {
                    break;
                }
            }
            return media;
        }

        public static Chunk DecodeChunk(Stream stream, DataReader reader, FormatDef format)
        {
            Chunk chunk = new Chunk();
            chunk.Offset = stream.Position;
            chunk.ID = reader.ReadUInt32LE();
            chunk.Size = reader.ReadUInt32BE();
            uint datasize = chunk.Size - 8;
            chunk.Data = new OverlayStream(stream, stream.Position, datasize);
            chunk.Decode(format);
            stream.Position = chunk.Offset + chunk.Size;
            return chunk;
        }
    }
}
