using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using QiHe.CodeLib;

namespace QiHe.RealMedia
{
    public class Writer
    {
        public static void Save(string file, MediaFile media, FormatDef format)
        {
            using (FileStream stream = File.Create(file))
            {
                DataWriter writer = new DataWriter(stream);

                //CalculateSize(media, format);
                foreach (Chunk chunk in media.Chunks)
                {
                    chunk.Encode(writer, format);
                }
            }
        }

        public static void CalculateSize(MediaFile media, FormatDef format)
        {
            uint offset = 0;
            foreach (Chunk chunk in media.Chunks)
            {
                chunk.Offset = offset;
                uint size = chunk.CalculateSize(format);
                offset += size;
            }
            LinkChunks(media);
        }

        private static void LinkChunks(MediaFile media)
        {
            Chunk prop = null;
            Chunk data = null;
            Chunk index = null;
            foreach (Chunk chunk in media.Chunks)
            {
                if (chunk.Name == "PROP")
                {
                    prop = chunk;
                }
                else if (chunk.Name == "DATA")
                {
                    if (data == null)
                    {
                        prop["data_offset"] = (uint)chunk.Offset;
                    }
                    else
                    {
                        data["next_data_header"] = (uint)chunk.Offset;
                    }
                    data = chunk;
                }
                else if (chunk.Name == "INDX")
                {
                    if (index == null)
                    {
                        prop["index_offset"] = (uint)chunk.Offset;
                    }
                    else
                    {
                        index["next_index_header"] = (uint)chunk.Offset;
                    }
                    index = chunk;
                }
            }
        }
    }
}
