using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using QiHe.CodeLib;

namespace QiHe.RealMedia
{
    public partial class RecordDef
    {
        public RecordDef()
        {
            Fields = new List<FieldDef>();
        }

        public Record Decode(DataReader reader, FormatDef format)
        {
            Record record = new Record();
            record.Name = this.Name;
            bool hasError = false;
            foreach (FieldDef field in Fields)
            {
                try
                {
                    record[field.Name] = field.Decode(reader, record, format, out hasError);
                    if (hasError)
                    {
                        record.HasError = true;
                    }
                }
                catch (Exception error)
                {
                    record[field.Name] = String.Format("Decode error: {0}", error.Message);
                    record.HasError = true;
                    break;
                }
            }
            return record;
        }


        public int CalculateSize(Record record, FormatDef format)
        {
            int size = 0;
            foreach (FieldDef field in Fields)
            {
                size += field.CalculateSize(record, format);
            }
            return size;
        }

        public void Encode(DataWriter writer, Record record, FormatDef format)
        {
            foreach (FieldDef field in Fields)
            {
                field.Encode(writer, record, format);
            }
        }
    }
}
