using System;
using System.Collections.Generic;
using System.Text;

namespace QiHe.RealMedia
{
    public partial class Process
    {
        public static void RebuildIndex(MediaFile media)
        {
            uint offset = 0;
            Chunk prop = null;
            IEnumerable<Record> packets = null;
            List<Chunk> indexChunks = new List<Chunk>();
            Dictionary<ushort, List<Record>> indexRecords = new Dictionary<ushort, List<Record>>();
            foreach (Chunk chunk in media.Chunks)
            {
                if (chunk.Name == "PROP")
                {
                    prop = chunk;
                }
                else if (chunk.Name == "MDPR")
                {
                    ushort stream_number = (ushort)chunk["stream_number"];
                    indexRecords[stream_number] = new List<Record>();
                }
                else if (chunk.Name == "DATA")
                {
                    offset = (uint)chunk.Offset;
                    packets = (IEnumerable<Record>)chunk["packets"];
                }
                else if (chunk.Name == "INDX")
                {
                    indexChunks.Add(chunk);
                }
            }
            prop["num_streams"] = (ushort)indexRecords.Count;
            //remove old index chunks
            foreach (Chunk chunk in indexChunks)
            {
                media.Chunks.Remove(chunk);
            }

            //collect key frame indice
            offset += 18;
            uint count = 0;
            Dictionary<ushort, uint> timestamps = new Dictionary<ushort, uint>();
            foreach (Record packet in packets)
            {
                byte flags = (byte)packet["flags"];
                if ((flags & 2) != 0) // key frame
                {
                    ushort stream_number = (ushort)packet["stream_number"];
                    uint timestamp = (uint)packet["timestamp"];
                    if (!timestamps.ContainsKey(stream_number) ||
                        timestamp != timestamps[stream_number])
                    {
                        timestamps[stream_number] = timestamp;
                        Record index = new Record();
                        index["object_version"] = (ushort)0;
                        index["timestamp"] = timestamp;
                        index["offset"] = offset;
                        index["packet_index"] = count;
                        indexRecords[stream_number].Add(index);
                    }
                }
                ushort length = (ushort)packet["length"];
                offset += length;
                count++;
            }

            //create new index chunks
            foreach (KeyValuePair<ushort, List<Record>> indice in indexRecords)
            {
                Chunk chunk = new Chunk();
                chunk.ID = 1480871497;
                chunk.Name = "INDX";
                Record record = new Record();
                record.Name = "INDX";
                record["object_version"] = (ushort)0;
                record["num_indices"] = (uint)indice.Value.Count;
                record["stream_number"] = indice.Key;
                record["next_index_header"] = (uint)0;
                record["indices"] = indice.Value;
                chunk.Record = record;
                media.Chunks.Add(chunk);
            }
        }
    }
}
